"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransactionManager = void 0;
const instancefactory_1 = require("../main/instancefactory");
const aopfactory_1 = require("../main/aopfactory");
const transactionadvice_1 = require("./transactionadvice");
const errorfactory_1 = require("../tools/errorfactory");
const mysqltransaction_1 = require("./mysqltransaction");
const dbmanager_1 = require("./dbmanager");
const oracletransaction_1 = require("./oracletransaction");
const application_1 = require("../tools/application");
const mssqltransaction_1 = require("./mssqltransaction");
const typeormtransaction_1 = require("./typeormtransaction");
const relaentransaction_1 = require("./relaentransaction");
const threadlocal_1 = require("../tools/threadlocal");
class TransactionManager {
    static init(cfg) {
        this.initAdvice();
        //transaction实例名
        this.transactionMdl = cfg.transaction || '__noomi_transaction';
        //隔离级
        if (cfg.isolation_level && typeof cfg.isolation_level === 'number') {
            this.isolationLevel = cfg.isolation_level;
        }
        //添加transaction到实例工厂，已存在则不再添加
        let tn = this.transactionMdl;
        //事务类
        let clazz;
        if (tn) {
            let ins = instancefactory_1.InstanceFactory.getInstance(tn);
            if (ins === null) {
                switch (cfg.product) {
                    case "relaen":
                        clazz = relaentransaction_1.RelaenTransaction;
                        break;
                    case "mysql":
                        clazz = mysqltransaction_1.MysqlTransaction;
                        break;
                        break;
                    case "mssql":
                        clazz = mssqltransaction_1.MssqlTransaction;
                        break;
                    case "oracle":
                        clazz = oracletransaction_1.OracleTransaction;
                        break;
                    case 'typeorm': //typeorm
                        clazz = typeormtransaction_1.TypeormTransaction;
                        this.transactionOption = {};
                        //设置隔离级别
                        if (this.isolationLevel !== 0) {
                            switch (TransactionManager.isolationLevel) {
                                case 1:
                                    this.transactionOption.isolationLevel = "READ UNCOMMITTED";
                                    break;
                                case 2:
                                    this.transactionOption.isolationLevel = "READ COMMITTED";
                                    break;
                                case 3:
                                    this.transactionOption.isolationLevel = "REPEATABLE READ";
                                    break;
                                case 4:
                                    this.transactionOption.isolationLevel = "SERIALIZABLE";
                            }
                        }
                        break;
                }
                instancefactory_1.InstanceFactory.addInstance({
                    name: tn,
                    class: clazz,
                    singleton: false
                });
            }
        }
    }
    /**
     * 注册事务，在addInstance时正式添加到aop
     * @param className     类名
     * @param methodName    方法名或方法名串，支持通配符"*"
     */
    static registTransaction(className, methodName) {
        let arr = typeof methodName === 'string' ? [methodName] : methodName;
        if (this.registTransactionMap.has(className)) {
            arr = this.registTransactionMap.get(className).concat(arr);
        }
        this.registTransactionMap.set(className, arr);
    }
    /**
     * 处理实例事务，用于把事务注册map的事务添加到事务aop
     * @param instanceName      实例名
     * @param className         类名
     */
    static handleInstanceTranstraction(instanceName, className) {
        if (!this.registTransactionMap.has(className)) {
            return;
        }
        const methods = this.registTransactionMap.get(className);
        if (methods) {
            this.addTransaction(instanceName, methods);
        }
        //删除已处理的class
        this.registTransactionMap.delete(className);
    }
    /**
     * 添加为事务
     * @param instance      实例 或 类
     * @param methodName    方法名
     */
    static addTransaction(instance, methodName) {
        let pc = aopfactory_1.AopFactory.getPointcutById(this.pointcutId);
        let name = typeof instance === 'string' ? instance : instance.__instanceName;
        //数组
        if (Array.isArray(methodName)) {
            for (let n of methodName) {
                let expr = '^' + name + '.' + n + '$';
                aopfactory_1.AopFactory.addExpression('TransactionAdvice', this.pointcutId, expr);
            }
        }
        else {
            let expr = '^' + name + '.' + methodName + '$';
            aopfactory_1.AopFactory.addExpression('TransactionAdvice', this.pointcutId, expr);
        }
    }
    /**
     * 获取transaction
     * @param newOne    如果不存在，是否新建
     * @return          transacton
     */
    static async get(newOne) {
        let tr;
        //得到当前执行异步id
        let id = threadlocal_1.NoomiThreadLocal.getThreadId();
        if (!id && newOne) {
            id = threadlocal_1.NoomiThreadLocal.newThreadId();
        }
        if (!id) {
            return null;
        }
        if (this.transactionMap.has(id)) {
            tr = this.transactionMap.get(id);
        }
        else if (newOne) { //父亲对象不存在，则新建
            tr = instancefactory_1.InstanceFactory.getInstance(this.transactionMdl, [id]);
            // 保存到transaction map
            this.transactionMap.set(id, tr);
        }
        return tr;
    }
    /**
     * 删除事务
     * @param tr    事务
     */
    static del(tr) {
        this.transactionMap.delete(tr.id);
    }
    /**
     * 获取connection
     */
    static getConnection(id) {
        if (!id) {
            id = threadlocal_1.NoomiThreadLocal.getThreadId();
        }
        if (!this.transactionMap.has(id)) {
            return null;
        }
        let tr = this.transactionMap.get(id);
        return tr.connection;
    }
    /**
     * 释放连接
     * @param tr
     */
    static async releaseConnection(tr) {
        await dbmanager_1.DBManager.getConnectionManager().release(tr.connection);
    }
    /**
     * 解析实例配置文件
     * @param path      文件路径
     * @param mdlPath   模型路径
     */
    static parseFile(path) {
        //读取文件
        let jsonStr = application_1.App.fs.readFileSync(path, 'utf-8');
        let json = null;
        try {
            json = application_1.App.JSON.parse(jsonStr);
        }
        catch (e) {
            throw new errorfactory_1.NoomiError("2800") + '\n' + e;
        }
        this.init(json);
    }
    /**
     * 初始化transaction advice
     * @since 1.0.0
     */
    static initAdvice() {
        aopfactory_1.AopFactory.registPointcut({
            className: 'TransactionAdvice',
            id: this.pointcutId
        });
        aopfactory_1.AopFactory.registAdvice({
            pointcutId: this.pointcutId,
            type: 'before',
            method: 'before',
            className: 'TransactionAdvice'
        });
        aopfactory_1.AopFactory.registAdvice({
            pointcutId: this.pointcutId,
            type: 'after-return',
            method: 'afterReturn',
            className: 'TransactionAdvice'
        });
        aopfactory_1.AopFactory.registAdvice({
            pointcutId: this.pointcutId,
            type: 'after-throw',
            method: 'afterThrow',
            className: 'TransactionAdvice'
        });
        aopfactory_1.AopFactory.registAspect('TransactionAdvice');
        //添加Aspect
        instancefactory_1.InstanceFactory.addInstance({
            name: this.aspectName,
            class: transactionadvice_1.TransactionAdvice,
            singleton: true
        });
    }
}
exports.TransactionManager = TransactionManager;
/**
 * transaction map，用于维护事务，键为事务id，值为事务对象
 */
TransactionManager.transactionMap = new Map();
/**
 * 切点名
 */
TransactionManager.pointcutId = '__NOOMI_TX_POINTCUT';
/**
 * 切面名
 */
TransactionManager.aspectName = '__NOOMI_TX_ASPECT';
/**
 * 隔离级 1read uncommited 2read commited 3repeatable read 4serializable
 */
TransactionManager.isolationLevel = 0;
/**
 * 事务注册map，键为事务类名，值为[methodName1,methodName2,...]
 */
TransactionManager.registTransactionMap = new Map();
//# sourceMappingURL=transactionmanager.js.map