"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AopPointcut = exports.AopFactory = void 0;
const instancefactory_1 = require("./instancefactory");
const aopproxy_1 = require("./aopproxy");
const errorfactory_1 = require("../tools/errorfactory");
const transactionmanager_1 = require("../database/transactionmanager");
const util_1 = require("../tools/util");
/**
 * aop 切点类
 */
class AopPointcut {
    /**
     * 构造器
     * @param id            切点id(实例内唯一)
     * @param instanceName  实例名
     * @param expressions   该切点对应的表达式数组，表达式为正则表达式串
     */
    constructor(id, instanceName, expressions) {
        /**
         * 表达式数组（正则表达式）
         */
        this.expressions = [];
        /**
         * 通知数组
         */
        this.advices = [];
        this.id = id;
        this.instanceName = instanceName;
        if (expressions) {
            this.addExpression(expressions);
        }
    }
    /**
     * 匹配方法是否满足表达式
     * @param instanceName  实例名
     * @param methodName    待检测方法
     * @returns             匹配返回true，否则返回false
     */
    // public match(instanceName:string,methodName:string):boolean{
    //     for(let exp of this.expressions){
    //         if(exp.test(instanceName + '.' + methodName)){
    //             return true;
    //         }
    //     }
    //     return false;
    // }
    /**
     * 给切点添加通知
     * @param advice    通知对象
     */
    addAdvice(advice) {
        this.advices.push(advice);
    }
    /**
     * 添加表达式串
     * @param expr  表达式串或数组
     */
    addExpression(expr) {
        if (!Array.isArray(expr)) {
            expr = [expr];
        }
        expr.forEach((item) => {
            if (typeof item !== 'string') {
                throw new errorfactory_1.NoomiError("2001");
            }
            this.expressions.push(util_1.Util.toReg(item));
        });
    }
}
exports.AopPointcut = AopPointcut;
/**
 * Aop工厂
 * 用于管理所有切面、切点
 */
class AopFactory {
    /**
     * 注册切面
     * @param className     切面类名
     * @since 1.0.0
     */
    static registAspect(className) {
        if (!this.registAspectMap.has(className)) {
            return;
        }
        this.registAspectMap.get(className).isAspect = true;
    }
    /**
     * 注册切点
     * @param cfg   pointcut配置
     * @since 1.0.0
     */
    static registPointcut(cfg) {
        let pc = this.getRegistPointcut(cfg.className, cfg.id, true);
        if (cfg.expressions) {
            pc.expressions = pc.expressions.concat(cfg.expressions);
        }
    }
    /**
     * 注册advice
     * @param cfg   advice配置
     * @since 1.0.0
     */
    static registAdvice(cfg) {
        if (!this.registAspectMap.has(cfg.className)) {
            return;
        }
        let pc = this.registAspectMap.get(cfg.className)[cfg.pointcutId];
        if (!pc) {
            return;
        }
        delete cfg.className;
        pc.advices.push(cfg);
    }
    /**
     * 从registAspectMap中获取注册的pointcut配置
     * @param className     类名
     * @param pointcutId    切点id
     * @param create        如果不存在，是否创建，如果为true，则创建，默认false
     * @returns             pointcut配置项
     * @since 1.0.0
     */
    static getRegistPointcut(className, pointcutId, create) {
        let pc;
        if (this.registAspectMap.has(className)) {
            let obj = this.registAspectMap.get(className);
            pc = obj[pointcutId];
        }
        else if (create) {
            this.registAspectMap.set(className, {});
        }
        if (!pc && create) { //新建pointcut配置项
            let obj = this.registAspectMap.get(className);
            pc = {
                advices: [],
                expressions: []
            };
            obj[pointcutId] = pc;
        }
        return pc;
    }
    /**
     * 处理实例切面
     * 把注册的切面添加到切面列表
     * @param instanceName  实例名
     * @param className     类名
     * @since 1.0.0
     */
    static handleInstanceAspect(instanceName, className) {
        if (!this.registAspectMap.has(className)) {
            return;
        }
        let cfg = this.registAspectMap.get(className);
        if (!cfg.isAspect) { //非aspect，不处理
            return;
        }
        delete cfg.isAspect;
        Object.keys(cfg).forEach(item => {
            let o = cfg[item];
            //新建pointcut
            let pc = new AopPointcut(item, instanceName, o.expressions);
            //加入切点集
            this.pointcuts.set(instanceName + '.' + item, pc);
            //为切点添加advice
            o.advices.forEach(item1 => {
                //添加advice
                pc.addAdvice(item1);
            });
        });
        //删除已处理的class
        this.registAspectMap.delete(className);
    }
    /**
     * 为切点添加表达式
     * @param className     切点类名
     * @param pointcutId    切点id
     * @param expression    表达式或数组
     */
    static addExpression(className, pointcutId, expression) {
        let pc = this.pointcuts.get(pointcutId);
        if (!pc) {
            return;
        }
        pc.addExpression(expression);
    }
    /**
     * 为所有aop匹配的方法设置代理
     */
    static proxyAll() {
        if (!this.pointcuts || this.pointcuts.size === 0) {
            return;
        }
        let pcArr = this.pointcuts.values();
        //遍历pointcut
        let pc;
        for (let ins of instancefactory_1.InstanceFactory.getFactory()) {
            this.proxyOne(ins[0]);
        }
    }
    /**
     * 为某个类设置代理
     * @param instanceName  实例名
     * @param clazz         类
     */
    static proxyOne(instanceName) {
        if (!this.pointcuts || this.pointcuts.size === 0) {
            return;
        }
        const clazz = instancefactory_1.InstanceFactory.getInstanceCfg(instanceName).class;
        //遍历pointcut
        let pc;
        for (pc of this.pointcuts.values()) {
            let reg;
            //遍历expression
            for (reg of pc.expressions) {
                Object.getOwnPropertyNames(clazz.prototype).forEach(key => {
                    //给方法设置代理，constructor 不需要代理
                    if (key === 'constructor' || typeof clazz.prototype[key] !== 'function') {
                        return;
                    }
                    let method = clazz.name + '.' + key;
                    if (reg.test(method)) {
                        if (this.proxyMethodMap.has(method)) {
                            let arr = this.proxyMethodMap.get(method);
                            //不重复代理
                            if (!arr.includes(pc)) {
                                arr.push(pc);
                            }
                        }
                        else {
                            //首次需要代理方法
                            clazz.prototype[key] = aopproxy_1.AopProxy.invoke(instanceName, key);
                            //设置方法对应pointcut数组
                            this.proxyMethodMap.set(method, [pc]);
                        }
                    }
                });
            }
        }
    }
    /**
     * 根据id获取切点
     * @param pointcutId    切点id
     * @returns             切点对象
     */
    static getPointcutById(pointcutId) {
        return this.pointcuts.get(pointcutId);
    }
    /**
     * 清理实例aop代理
     * @param instanceName  实例名
     */
    static clearProxy(instanceName) {
        for (let k of this.proxyMethodMap.keys()) {
            if (k.startsWith(instanceName + '.')) {
                this.proxyMethodMap.delete(k);
            }
        }
    }
    /**
     * 获取advices
     * @param instanceName  实例名
     * @param methodName    方法名
     * @return              {
     *                          before:[{instance:切面实例,method:切面方法},...]
     *                          after:[{instance:切面实例,method:切面方法},...]
     *                          return:[{instance:切面实例,method:切面方法},...]
     *                          throw:[{instance:切面实例,method:切面方法},...]
     *                      }
     */
    static getAdvices(className, methodName) {
        const pointcuts = this.proxyMethodMap.get(className + '.' + methodName);
        if (pointcuts.length === 0) {
            return null;
        }
        let beforeArr = [];
        let aroundArr1 = []; //前置around
        let aroundArr2 = []; //后置around
        let afterArr = [];
        let throwArr = [];
        let returnArr = [];
        let pointcut;
        let hasTransaction = false;
        for (pointcut of pointcuts) {
            if (pointcut.id === transactionmanager_1.TransactionManager.pointcutId) {
                hasTransaction = true;
                continue;
            }
            pointcut.advices.forEach(item => {
                let ins = instancefactory_1.InstanceFactory.getInstance(pointcut.instanceName);
                switch (item.type) {
                    case 'before':
                        beforeArr.push({
                            instance: ins,
                            method: item.method
                        });
                        return;
                    case 'after':
                        afterArr.push({
                            instance: ins,
                            method: item.method
                        });
                        return;
                    case 'around':
                        aroundArr1.push({
                            instance: ins,
                            method: item.method
                        });
                        aroundArr2.push({
                            instance: ins,
                            method: item.method
                        });
                        return;
                    case 'after-return':
                        returnArr.push({
                            instance: ins,
                            method: item.method
                        });
                        return;
                    case 'after-throw':
                        throwArr.push({
                            instance: ins,
                            method: item.method
                        });
                }
            });
        }
        beforeArr = aroundArr1.concat(beforeArr);
        afterArr = afterArr.concat(aroundArr2);
        return {
            hasTransaction: hasTransaction,
            before: beforeArr,
            after: afterArr,
            throw: throwArr,
            return: returnArr
        };
    }
}
exports.AopFactory = AopFactory;
/**
 * 切点map，用于存储所有切点
 * key:切面实例名 + '.' + 切点名
 */
AopFactory.pointcuts = new Map();
/**
 * 已代理方法map，键为instanctName.methodName，避免重复代理
 * @since 0.4.4
 */
AopFactory.proxyMethodMap = new Map();
/**
 * 注册切面map，键为className,值为
 *          {
 *              isAspect:true,  //避免用了pointcut，但是未使用Aspect注解
 *              pointCutId1:{expressions:Array<string>,advices:{type:类型,method:方法名}},
 *              ...
 *          }
 */
AopFactory.registAspectMap = new Map();
//# sourceMappingURL=aopfactory.js.map