"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InstanceFactory = void 0;
const errorfactory_1 = require("../tools/errorfactory");
const util_1 = require("../tools/util");
const application_1 = require("../tools/application");
const filewatcher_1 = require("../tools/filewatcher");
const transactionmanager_1 = require("../database/transactionmanager");
const filterfactory_1 = require("../web/filterfactory");
const routefactory_1 = require("./route/routefactory");
const webafterhandler_1 = require("../web/webafterhandler");
const aopfactory_1 = require("./aopfactory");
/**
 * 实例工厂
 * @remarks
 * 用于管理所有的实例对象
 */
class InstanceFactory {
    /**
     * 工厂初始化
     * @param config    配置项
     */
    static async init(config) {
        await this.parse(config);
    }
    /**
     * 添加单例到工厂
     * @param cfg       实例配置对象
     * @param replace   替换之前的实例
     * @returns         undefined或添加的实例
     */
    static addInstance(cfg) {
        cfg.class.prototype.__instanceName = cfg.name;
        this.factory.set(cfg.name, {
            class: cfg.class,
            singleton: cfg.singleton !== false
        });
        //依赖实例名的相关处理
        aopfactory_1.AopFactory.handleInstanceAspect(cfg.name, cfg.class.name);
        routefactory_1.RouteFactory.handleInstanceRoute(cfg.name, cfg.class.name);
        transactionmanager_1.TransactionManager.handleInstanceTranstraction(cfg.name, cfg.class.name);
        filterfactory_1.FilterFactory.handleInstanceFilter(cfg.name, cfg.class.name);
        webafterhandler_1.WebAfterHandler.handleInstanceHandler(cfg.name, cfg.class.name);
        //因为可能切面尚未添加到实例工厂，延迟aop代理
        setImmediate(() => {
            aopfactory_1.AopFactory.clearProxy(cfg.class.prototype.__instanceName);
            aopfactory_1.AopFactory.proxyOne(cfg.class.prototype.__instanceName);
        });
    }
    /**
     * 注入
     * @param targetClassName   目标类名
     * @param propName          注入属性名
     * @param injectName        注入实例名
     */
    static inject(targetClassName, propName, injectName) {
        if (this.injectMap.has(targetClassName)) {
            let arr = this.injectMap.get(targetClassName);
            //不重复注入，如果属性已注入，则修改注入名，否则增加到数组
            let r = arr.find(item => item['propName'] === propName);
            if (r) {
                r['injectName'] = injectName;
            }
            else {
                arr.push({ propName: propName, injectName: injectName });
            }
        }
        else {
            this.injectMap.set(targetClassName, [{ propName: propName, injectName: injectName }]);
        }
    }
    /**
     * 获取实例
     * @param name  实例名
     * @param param 参数数组
     * @returns     实例化的对象或null
     */
    static getInstance(name, param) {
        let ins = this.factory.get(name);
        if (!ins) {
            return null;
        }
        if (ins.singleton && ins.instance) {
            if (this.injectMap.has(ins.class.name)) {
                //重新注入
                this.injectMap.get(ins.class.name).forEach((item) => {
                    ins.instance[item['propName']] = this.getInstance(item['injectName']);
                });
            }
            return ins.instance;
        }
        else {
            let mdl = ins.class;
            param = param || ins.params || [];
            let instance = Reflect.construct(mdl, param);
            if (this.injectMap.has(ins.class.name)) {
                this.injectMap.get(ins.class.name).forEach((item) => {
                    instance[item['propName']] = this.getInstance(item['injectName']);
                });
            }
            if (ins.singleton) {
                ins.instance = instance;
            }
            return instance;
        }
    }
    /**
     * 获取实例配置对象
     * @param instanceName  实例名
     * @returns             实例配置对象
     */
    static getInstanceCfg(instanceName) {
        return this.factory.get(instanceName);
    }
    /**
     * 执行实例的一个方法
     * @param instancee     实例名或实例对象
     * @param methodName    方法名
     * @param params        参数数组
     * @param func          方法(与methodName二选一)
     * @returns             方法对应的结果
     */
    static exec(instance, methodName, params, func) {
        //实例名，需要得到实例对象
        let instanceName = '';
        if (instance && typeof instance === 'string') {
            instanceName = instance;
            instance = this.getInstance(instance);
        }
        //实例不存在
        if (!instance) {
            throw new errorfactory_1.NoomiError("1001", instanceName);
        }
        func = func || instance[methodName];
        //方法不存在
        if (!func) {
            throw new errorfactory_1.NoomiError("1010", methodName);
        }
        return func.apply(instance, params);
    }
    /**
     * @exclude
     * 处理配置对象
     * @param
     */
    static async parse(path) {
        if (Array.isArray(path)) {
            for (let p of path) {
                await handle(p);
            }
        }
        else {
            await handle(path);
        }
        /**
         * 处理instance路径
         * @param path  待解析路径
         */
        async function handle(path) {
            const basePath = process.cwd();
            let pathArr = path.split('/');
            let pa = [basePath];
            let handled = false; //是否已处理
            for (let i = 0; i < pathArr.length - 1; i++) {
                const p = pathArr[i];
                if (p.indexOf('*') === -1 && p !== "") {
                    pa.push(p);
                }
                else if (p === '**') { //所有子孙目录
                    handled = true;
                    if (i < pathArr.length - 2) {
                        throw new errorfactory_1.NoomiError('1000');
                    }
                    await handleDir(pa.join('/'), pathArr[pathArr.length - 1], true);
                }
            }
            if (!handled) {
                await handleDir(pa.join('/'), pathArr[pathArr.length - 1]);
            }
            /**
             * 处理子目录
             * @param dirPath   目录地址
             * @param fileExt   文件后缀
             * @param deep      是否深度处理
             */
            async function handleDir(dirPath, fileExt, deep) {
                const dir = application_1.App.fs.readdirSync(dirPath, { withFileTypes: true });
                let fn = fileExt;
                let reg = util_1.Util.toReg(fn, 3);
                //添加 file watcher
                if (application_1.App.openWatcher) {
                    filewatcher_1.FileWatcher.addDir(dirPath, filewatcher_1.EWatcherType.DYNAMIC);
                }
                for (const dirent of dir) {
                    if (dirent.isDirectory()) {
                        if (deep) {
                            handleDir(application_1.App.path.resolve(dirPath, dirent.name), fileExt, deep);
                        }
                    }
                    else if (dirent.isFile()) {
                        // @Instance注解方式文件，自动执行instance创建操作
                        if (reg.test(dirent.name)) {
                            await Promise.resolve().then(() => require(application_1.App.path.resolve(dirPath, dirent.name)));
                        }
                    }
                }
            }
        }
    }
    /**
     * 获取instance工厂
     * @returns     实例工厂
     */
    static getFactory() {
        return this.factory;
    }
}
exports.InstanceFactory = InstanceFactory;
/**
 * 实例工厂map，存放所有实例对象
 */
InstanceFactory.factory = new Map();
/**
 * 注入依赖map  键为注入类名，值为数组，数组元素为{className:类名,propName:属性名}
 * @since 0.4.4
 */
InstanceFactory.injectMap = new Map();
//# sourceMappingURL=instancefactory.js.map