"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseModel = void 0;
const validator_1 = require("./validator");
const noomimodeltip_1 = require("../locales/noomimodeltip");
const application_1 = require("./application");
const util_1 = require("./util");
/**
 * 基础模型
 */
class BaseModel {
    /**
     * 构造器
     * @param nullArr   对应路由方法的空校验字段map
     */
    constructor(nullArr) {
        this.__nullCheckArr = nullArr;
    }
    /**
     * 转换和校验，返回数据类型或校验不正确的属性消息集合
     */
    __handle() {
        let errObj = {};
        //空校验
        let r;
        if (this.__nullCheckArr) {
            for (let p of this.__nullCheckArr) {
                r = validator_1.Validator.validate('nullable', this[p], null);
                if (!r) {
                    errObj[p] = util_1.Util.compileString(noomimodeltip_1.NoomiModelTip[application_1.App.language]['nullable'], []);
                }
            }
        }
        for (let p of Object.keys(this)) {
            //空校验异常或__开头的属性不处理
            if (errObj[p] || p.startsWith('__')) {
                continue;
            }
            //类型转换
            if (!this.__transform(p)) {
                errObj[p] = noomimodeltip_1.NoomiModelTip[application_1.App.language][this.constructor['__typeMap'].get(p)];
            }
            else {
                let r = this.__validate(p);
                if (r !== null) {
                    errObj[p] = r;
                }
            }
        }
        return Object.getOwnPropertyNames(errObj).length === 0 ? null : errObj;
    }
    /**
     * 验证
     * @param name  属性名
     * @returns     null或字符串(表示验证异常)
     */
    __validate(name) {
        const validators = this.constructor['__validatorMap'].get(name);
        if (!validators) {
            return null;
        }
        let value = this[name];
        for (let vn of Object.keys(validators)) {
            if (validator_1.Validator.hasValidator(vn)) {
                let r = validator_1.Validator.validate(vn, value, validators[vn]);
                if (!r) {
                    return util_1.Util.compileString(noomimodeltip_1.NoomiModelTip[application_1.App.language][vn], validators[vn]);
                }
            }
            else if (this[vn] && typeof this[vn] === 'function') { //模型自定义校验器
                let r = this[vn](value, validators[vn]);
                if (r !== null) {
                    return r;
                }
            }
        }
        return null;
    }
    /**
     * 数据格式转换
     * @param name  属性名
     * @returns     true 转换成功 false转换失败
     */
    __transform(name) {
        const type = this.constructor['__typeMap'].get(name);
        if (!type) {
            return true;
        }
        let v = this[name];
        const tp = typeof v;
        //非字符串，需要去掉两端空格
        if (type !== 'string' && tp === 'string') {
            v = v.trim();
            //非字符串，且为''，则删除
            if (v === '') {
                delete this[name];
                return true;
            }
        }
        //类型不为string则不转换
        if (tp === 'string') {
            switch (type) {
                case 'int': //整数
                    if (/(^0$)|(^[1-9]\d*$)/.test(v)) {
                        v = parseInt(v);
                    }
                    else {
                        return false;
                    }
                    break;
                case 'float': //小数
                    if (/^\d+(\.?\d+)?$/.test(v)) {
                        v = parseFloat(v);
                    }
                    else {
                        return false;
                    }
                    break;
                case 'boolean': //bool
                    if (v === 'true') {
                        v = true;
                    }
                    else if (v === 'false') {
                        v = false;
                    }
                    else {
                        return false;
                    }
                    break;
                case 'array': //数组类型
                    try {
                        v = util_1.Util.eval(v);
                        if (!Array.isArray(v)) {
                            return false;
                        }
                    }
                    catch (e) {
                        return false;
                    }
                    break;
                case 'object': //object类型
                    try {
                        v = util_1.Util.eval('(' + v + ')');
                        if (typeof v !== 'object') {
                            return false;
                        }
                    }
                    catch (e) {
                        return false;
                    }
                    break;
                default: //字符串，不处理
            }
        }
        this[name] = v;
        return true;
    }
    /**
     * 设置校验器
     * @param name          属性名
     * @param validators    验证器
     */
    static __setValidator(name, validators) {
        if (!this.__validatorMap.has(name)) {
            this.__validatorMap.set(name, validators);
        }
    }
    /**
     * 设置数据类型
     * @param name      属性名
     * @param type      属性类型
     */
    static __setType(name, type) {
        if (!this.__typeMap.has(name)) {
            this.__typeMap.set(name, type);
        }
    }
}
exports.BaseModel = BaseModel;
/**
 * 校验器map
 * key:propName,value:校验器数组
 */
BaseModel.__validatorMap = new Map();
/**
 * 数据类型map
 * key:propName,value:数据类型
 */
BaseModel.__typeMap = new Map();
//# sourceMappingURL=model.js.map