"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PostFormHandler = void 0;
const application_1 = require("../tools/application");
const util_1 = require("../tools/util");
/**
 * form 参数处理器
 */
class PostFormHandler {
    /**
     * @param boundary      属性分隔符
     * @param tmpDir        文件临时存储路径
     */
    constructor(boundary, tmpDir) {
        if (boundary) {
            this.boundary = Buffer.from('--' + boundary);
        }
        this.saveDir = tmpDir;
        this.returnObj = {};
    }
    /**
     * 处理buf
     * @param buf  新缓冲区
     */
    handleBuf(buf) {
        //获取换行符，首次执行时处理
        this.getLineBreak(buf);
        let buf1 = this.buf ? Buffer.concat([this.buf, buf]) : buf;
        let ind = buf1.indexOf(this.boundary);
        if (ind !== -1) { //有分隔符
            //循环处理所有数据项
            while ((ind = buf1.indexOf(this.boundary)) !== -1) {
                if (this.valueStarted) { //上一帧值已经开始，尚未传完
                    let buf2 = buf1.subarray(0, ind - this.lineBreak.length);
                    if (this.isFile) { //文件
                        if (this.value.fileName) {
                            application_1.App.fs.writeFileSync(this.value.path, buf2, { flag: 'a+' });
                        }
                    }
                    else { //普通串
                        this.value = (this.value ? Buffer.concat([this.value, buf2]) : buf2).toString();
                    }
                    //属性值绑定到属性名，如果属性名已存在，则作为数组
                    if (this.returnObj.hasOwnProperty(this.propName)) {
                        //新建数组
                        if (!Array.isArray(this.returnObj[this.propName])) {
                            this.returnObj[this.propName] = [this.returnObj[this.propName]];
                        }
                        //把新值加入数组
                        this.returnObj[this.propName].push(this.value);
                    }
                    else {
                        this.returnObj[this.propName] = this.value;
                    }
                    //处理完当前属性，参数还原
                    this.propName = undefined;
                    this.value = undefined;
                    this.valueStarted = undefined;
                    this.isFile = undefined;
                    this.buf = buf1.subarray(ind + this.boundary.length + this.lineBreak.length);
                }
                else if (ind === 0) { //最开始为分隔符，直接去掉分隔符
                    this.buf = buf1.subarray(ind + this.boundary.length + this.lineBreak.length);
                }
                else { //上一帧无法构成完整数据项
                    this.buf = buf1;
                }
                //取属性
                this.handleNewProp();
                //更新buf1
                buf1 = this.buf;
            }
        }
        else if (this.valueStarted) { //无分隔符，值已开始，尚未结束
            if (this.isFile) { //文件
                if (this.buf && this.value.fileName) {
                    //直接把当前buf写文件
                    application_1.App.fs.writeFileSync(this.value.path, this.buf, { flag: 'a+' });
                }
                //保留当前帧，可能需要和下一帧进行合并为有效数据项
                this.buf = buf;
            }
            else { //不是文件，添加到buf，可能需要和下一帧合并为有效数据项
                this.buf = buf1;
            }
        }
        else { //无分隔符，值未开始，数据项信息不完整
            this.buf = buf1;
        }
    }
    /**
     * 获取换行符
     * @param buf   来源buffer
     * @returns
     */
    getLineBreak(buf) {
        if (this.lineBreak) {
            return;
        }
        this.lineBreak = buf.indexOf('\r\n') === -1 ? '\n' : '\r\n';
        let buf1 = buf;
        while (!this.boundary && buf1.length > 0) {
            let ind = buf1.indexOf(this.lineBreak);
            if (ind === -1) {
                return;
            }
            let buf2 = buf1.subarray(0, ind);
            let line = buf2.toString().trim();
            if (line.startsWith('--')) {
                this.boundary = Buffer.from(line);
                break;
            }
            buf1 = buf1.subarray(ind + this.lineBreak.length);
        }
    }
    /**
     * 从buffer读一行
     * @param buf   buf
     * @returns     行字符串
     */
    readLine() {
        if (!this.buf || this.buf.length === 0) {
            return null;
        }
        let index = this.buf.indexOf(this.lineBreak);
        if (index === -1) {
            return null;
        }
        let r = this.buf.subarray(0, index).toString();
        this.buf = this.buf.subarray(index + this.lineBreak.length);
        return r;
    }
    /**
     * 处理新属性
     * 包括属性名、文件类型，值前的空行
     */
    handleNewProp() {
        if (this.valueStarted) {
            return;
        }
        //取属性
        if (this.buf.length > 0 && !this.propName) {
            this.handleProp();
        }
        if (this.value && this.value.fileName) {
            if (this.buf.length > 0 && !this.value.fileType) {
                this.handleFileType();
            }
            if (this.value.fileType) {
                let r;
                while ((r = this.readLine()) !== null) {
                    if (r === '') {
                        this.valueStarted = true;
                        break;
                    }
                }
            }
        }
        else if (this.readLine() !== null) {
            this.valueStarted = true;
        }
    }
    /**
     * 处理文件类型
     */
    handleFileType() {
        let line = this.readLine();
        if (!line || line === '') {
            return;
        }
        this.value['fileType'] = line.substr(line.indexOf(':') + 1).trim();
    }
    /**
     * 处理属性名
     */
    handleProp() {
        let line;
        //空行不处理
        while ((line = this.readLine()) === '' && line)
            ;
        //无行可读
        if (!line) {
            return;
        }
        let arr = line.split(';');
        //数据项名
        let pn = arr[1].substr(arr[1].indexOf('=')).trim();
        this.propName = pn.substring(2, pn.length - 1);
        if (arr.length > 2) { //文件
            for (let i = 2; i < arr.length; i++) {
                let s = arr[i].trim();
                if (s.startsWith('filename=')) {
                    this.isFile = true;
                    let a1 = s.split('=');
                    //文件名为空，此项不存
                    if (a1[1] == '""') {
                        this.value = {};
                        return;
                    }
                    let fn = a1[1];
                    let fn1 = fn.substring(1, fn.length - 1).trim();
                    let fn2 = application_1.App.uuid.v1() + fn1.substr(fn1.lastIndexOf('.'));
                    //得到绝对路径
                    let filePath = util_1.Util.getAbsPath([this.saveDir, fn2]);
                    this.value = {
                        fileName: fn1,
                        path: filePath
                    };
                }
            }
        }
    }
    /**
     * 获取result
     * @returns     参数值对象
     */
    getResult() {
        return this.returnObj;
    }
}
exports.PostFormHandler = PostFormHandler;
//# sourceMappingURL=postformhandler.js.map