import { HttpRequest } from "../../web/httprequest";
import { HttpResponse } from "../../web/httpresponse";
import { IWebCacheObj } from "../../web/webcache";
/**
 * 路由配置类型
 */
interface IRouteCfg {
    /**
     * 路由路径
     */
    path?: string;
    /**
     * 类名
     */
    className?: string;
    /**
     * 命名空间
     */
    namespace?: string;
    /**
     * 路由类
     */
    clazz?: any;
    /**
     * 路由正则表达式
     */
    reg?: RegExp;
    /**
     * 该路由对应的实例名
     */
    instanceName?: string;
    /**
     * 该路由对应的实例方法
     */
    method?: string;
    /**
     * 路由执行结果数组
     */
    results?: Array<IRouteResult>;
}
/**
 * 路由结果类型
 * @since 0.0.6
 */
declare enum ERouteResultType {
    /**
     * 重定向
     */
    REDIRECT = "redirect",
    /**
     * 路由链,和redirect不同，浏览器地址不会改变
     */
    CHAIN = "chain",
    /**
     * 文件流，主要用于文件下载
     */
    STREAM = "stream",
    /**
     * 什么都不做
     */
    NONE = "none",
    /**
     * json数据,默认类型
     */
    JSON = "json"
}
/**
 * 路由结果类型
 */
interface IRouteResult {
    /**
     * 结果类型
     */
    type?: ERouteResultType;
    /**
     * 返回值，当返回值与value一致时，将执行此结果
     */
    value?: any;
    /**
     * 路径，type 为redirect 和 url时，必须设置
     */
    url?: string;
    /**
     * 参数名数组，当type为chain时，从当前路由对应类中获取参数数组对应的属性值并以参数对象传递到下一个路由
     */
    params?: Array<string>;
}
/**
 * 路由对象
 */
interface IRoute {
    /**
     * 路由对应实例
     */
    instance: any;
    /**
     * 路由对应方法名
     */
    method: string;
    /**
     * 路由处理结果集
     */
    results?: Array<IRouteResult>;
    /**
     * route 实例对应url路径
     * @since 0.4.7
     */
    path?: string;
    /**
     * 参数对象
     * @since 0.4.7
     */
    params?: object;
}
/**
 * 路由工厂类
 * 用于管理所有路由对象
 */
declare class RouteFactory {
    /**
     * 静态路由(不带通配符)路由集合
     */
    static routeMap: Map<string, IRouteCfg>;
    /**
     * 注册路由map，添加到实例工厂时统一处理
     */
    private static registRouteMap;
    /**
     * 注册路由
     * @param cfg   路由配置
     */
    static registRoute(cfg: IRouteCfg): void;
    /**
     * 处理实例路由
     * 把注册路由添加到路由对象中
     * @param instanceName  实例名
     * @param className     类名
     */
    static handleInstanceRoute(instanceName: string, className: string): void;
    /**
     * 添加路由
     * @param path          路由路径，支持通配符*，需要method支持
     * @param className     对应类
     * @param method        方法，path中包含*，则不设置
     * @param results       路由处理结果集
     */
    private static addRoute;
    /**
     * 根据路径获取路由对象
     * @param path      url路径
     * @returns         路由对象
     */
    static getRoute(path: string): IRoute;
    /**
     * 路由方法执行
     * @param pathOrRoute   路径或路由
     * @param req           request 对象
     * @param res           response 对象
     * @param params        调用参数对象
     * @returns             0 正常 1异常
     */
    static handleRoute(route: IRoute, req: HttpRequest, res: HttpResponse, params?: object): Promise<number | IWebCacheObj | string>;
    /**
     * 处理路由结果
     * @param route     route对象
     * @param data      路由对应方法返回值
     */
    static handleResult(route: IRoute, data: any): Promise<number | IWebCacheObj | string>;
    /**
     * 处理一个路由结果
     * @param route         route对象
     * @param result        route result
     * @param data          路由执行结果
     * @returns             cache数据对象或0
     */
    static handleOneResult(route: IRoute, result: IRouteResult, data: any): Promise<IWebCacheObj | number | string>;
    /**
     * 处理异常信息
     * @param res   response 对象
     * @param e     异常
     * @deprecated  1.0.0
     */
    static handleException(res: HttpResponse, e: any): void;
}
export { RouteFactory, IRoute, IRouteCfg, IRouteResult, ERouteResultType };
