import { HttpRequest } from "../web/httprequest";
import { NCache } from "./ncache";
import { Session } from "../web/sessionfactory";
/**
 * 安全工厂
 * @remarks
 * 用于管理安全对象
 */
declare class SecurityFactory {
    /**
     * @exclude
     * 存储在session中的名字
     */
    static sessionName: string;
    /**
     * 数据表对象
     */
    static dbOptions: any;
    /**
     * 认证类型 0 session 1 token
     */
    static authType: number;
    /**
     * 数据存储类型，0内存 1redis
     */
    static saveType: number;
    /**
     * redis名，必须在redis.json中已定义,saveType=1时有效
     */
    static redis: string;
    /**
     * 最大size,saveType=0时有效
     */
    static maxSize: number;
    /**
     * 缓存对象
     */
    static cache: NCache;
    /**
     * 安全相关页面map
     */
    static securityPages: Map<string, string>;
    /**
     * 登录用户map
     */
    static users: Map<number, Array<number>>;
    /**
     * 组map
     */
    static groups: Map<number, Array<number>>;
    /**
     * @exclude
     * 缓存时用户key前缀
     */
    static USERKEY: string;
    /**
     * @exclude
     * 缓存时组key前缀
     */
    static GROUPKEY: string;
    /**
     * @exclude
     * 缓存时资源key前缀
     */
    static RESKEY: string;
    /**
     * @exclude
     * 用户id对应session键
     */
    static USERID: string;
    /**
     * 认证前url在session中的名字
     */
    static PRELOGIN: string;
    /**
     * users在cache的key
     */
    static redisUserKey: string;
    /**
     * groups在cache的key
     */
    static redisGroupKey: string;
    /**
     * resource在cache的key
     */
    static redisResourceKey: string;
    /**
     * 初始化配置
     * @config   配置项
     */
    static init(config: any): Promise<void>;
    /**
     * 添加用户组
     * @param userId    用户id
     * @param groupId   组id
     */
    static addUserGroup(userId: number, groupId: number): Promise<void>;
    /**
     * 添加用户组(多个组)
     * @param userId    用户id
     * @param groups    组id 数组
     */
    static addUserGroups(userId: number, groups: Array<number>, request?: HttpRequest): Promise<void>;
    /**
     * 添加组权限
     * @param groupId   组id
     * @param authId    权限id
     */
    static addGroupAuthority(groupId: number, authId: number): Promise<void>;
    /**
     * 更新组权限
     * @param groupId   组id
     * @param authIds   权限id数组
     */
    static updGroupAuths(groupId: number, authIds: Array<number>): Promise<void>;
    /**
     * 添加资源权限
     * @param url       资源url
     * @param authId    权限id
     */
    static addResourceAuth(url: string, authId: number): Promise<void>;
    /**
     * 添加资源权限(多个权限)
     * @param url       资源id
     * @param auths     权限id数组
     */
    static updResourceAuths(url: string, auths: Array<number>): Promise<void>;
    /**
     * 删除用户
     * @param userId    用户id
     * @param request   request对象
     */
    static deleteUser(userId: number, request?: HttpRequest): Promise<void>;
    /**
     * 删除用户组
     * @param userId    用户id
     * @param groupId   组id
     */
    static deleteUserGroup(userId: number, groupId: number): Promise<void>;
    /**
     * 删除组
     * @param groupId   组id
     */
    static deleteGroup(groupId: number): Promise<void>;
    /**
     * 删除组权限
     * @param groupId   组id
     * @param authId    权限id
     */
    static deleteGroupAuthority(groupId: number, authId: number): Promise<void>;
    /**
     * 删除资源
     * @param resourceId   资源id
     */
    static deleteResource(url: string): Promise<void>;
    /**
     * 删除资源权限
     * @param resourceId     资源id
     * @param authId    权限id
     */
    static deleteResourceAuthority(url: string, authId: number): Promise<void>;
    /**
     * 删除权限
     * @param authId    权限Id
     */
    static deleteAuthority(authId: number): Promise<void>;
    /**
     * 鉴权
     * @param url       资源url
     * @param session   session对象
     * @return          0 通过 1未登录 2无权限
     */
    static check(url: string, session: Session): Promise<number>;
    /**
     * 获取安全配置页面
     * @param name      配置项名
     * @return          页面url
     */
    static getSecurityPage(name: string): string;
    /**
     * 获取登录前页面
     * @param session   session对象
     * @return          page url
     */
    static getPreLoginInfo(request: HttpRequest): Promise<string>;
    /**
     * 设置认证前页面
     * @param session   session对象
     * @param page      page url
     */
    static setPreLoginInfo(session: Session, request: HttpRequest): Promise<void>;
    /**
     * @exclude
     * 文件解析
     * @param path 文件路径
     */
    static parseFile(path: any): Promise<void>;
}
export { SecurityFactory };
